GET_ASSISTANT_DATA_V1 = "v1/assistant/{id}"                          # GET - Gets assistant data
CREATE_ASSISTANT_V1 = "v1/assistant"                                 # POST - Creates a new assistant
UPDATE_ASSISTANT_V1 = "v1/assistant/{id}"                            # PUT -	Updates an assistant
DELETE_ASSISTANT_V1 = "v1/assistant/{id}"                            # DELETE - Deletes an assistant
BEGIN_CONVERSATION_V1 = "v1/assistant/text/begin"                    # POST - Begins a text conversation with the Globant Enterprise AI Assistant
SEND_TEXT_PROMPT_V1 = "v1/assistant/text"                            # POST - Sends a text prompt to the Globant Enterprise AI Assistant
SEND_CHAT_REQUEST_V1 = "v1/assistant/chat"                           # POST - Sends a chat request to the Globant Enterprise AI Assistant
GET_REQUEST_STATUS_V1 = "v1/assistant/request/{id}/status"           # GET - Retrieves the status of a request
CANCEL_REQUEST_V1 = "v1/assistant/request/{id}/cancel"               # POST - Cancels a request
GET_DATA_ANALYST_STATUS_V1 = "v1/assistant/{id}/status"              # GET - Gets the current status of the Data Analyst Assistant data set loading process.
EXTEND_DATA_ANALYST_DATASET_V1 = "v1/assistant/{id}/datasets/extend" # POST - Extends the data sets of an existing Data Analyst Assistant.